/* Establish cancellation handlers.
   Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <assert.h>
#include <pthread.h>

#include <pt-internal.h>

void
_pthread_cleanup_push (struct __pthread_cleanup *buffer,
											 void (*routine) (void *), void *arg)
{
	struct __pthread *self = (struct __pthread *) __pthread_self ();

	buffer->__routine = routine;
	buffer->__arg = arg;
	buffer->__next = self->cleanup;
	self->cleanup = buffer;
}

void _pthread_cleanup_pop (struct __pthread_cleanup *buffer, int execute)
{
	struct __pthread *self = (struct __pthread *) __pthread_self ();

	assert (buffer == self->cleanup);

	if (execute)
		buffer->__routine (buffer->__arg);
	self->cleanup = self->cleanup->__next;
}
