/* Initialize pthreads library.
   Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <assert.h>
#include <string.h>

#include <pt-internal.h>
#include <set-hooks.h>

/* The first (main) thread.  */
struct __pthread __pthread_main;

DEFINE_HOOK (__pthread_init, (struct __pthread *));

/* Initialize the pthreads library.  Returns a new stack pointer
   for the current thread, or NULL if no new stack is needed.  */
void *__pt_initialize (void)
{
	int err;

	err = __pthread_init (&__pthread_main);
	assert_perror (err);

	/* Copy attributes. */
	memcpy (&__pthread_main.attr, &__pt_default_attr,
			sizeof (struct __pthread_attr));

	/* Allocate stack.  */
	err = __pt_stack_alloc (&__pthread_main.attr.stackaddr,
			__pthread_main.attr.stacksize);
	assert_perror (err);

	/* We are the first (main) thread.  */
	__pthread_main.flags |= PTHREAD_COUNTED;

	RUN_HOOK (__pthread_init, (&__pthread_main));

	return __pt_stack_setup (&__pthread_main, NULL, NULL);
}
