/* Wait for thread termination.
   Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <errno.h>
#include <pthread.h>

#include <pt-internal.h>

/* Make calling thread wait for termination of thread THREAD.  Return
   the exit status of the thread in *STATUS.  */
int pthread_join (pthread_t thread, void **status)
{
#define thread ((struct __pthread *) thread)
	__pthread_mutex_lock (&thread->state_lock);

	while (thread->state != PTHREAD_EXITED && thread->state != PTHREAD_DETACHED)
		pthread_cond_wait (&thread->exited, &thread->state_lock);

	if (thread->state == PTHREAD_DETACHED) {
		__pthread_mutex_unlock (&thread->state_lock);
		return EINVAL;
	}

	if (status)
		*status = thread->status;

	__pthread_mutex_unlock (&thread->state_lock);
	__pthread_dealloc (thread);

	return 0;
#undef thread
}
