/* Thread-specific data access.
   Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <error.h>
#include <ihash.h>
#include <pthread.h>
#include <stdlib.h>
#include <spin-lock.h>

#include <pt-internal.h>

/* XXX */
#define __ihash_create	ihash_create
#define __ihash_add	ihash_add
#define __ihash_find	ihash_find

/* Store VALUE in the thread-specific data slot identified by KEY.  */
int pthread_setspecific (pthread_key_t key, const void *value)
{
	struct __pthread *self = (struct __pthread *) __pthread_self ();
	error_t err;

	if (self->specific == NULL) {
		self->specific = malloc (sizeof (struct pthread_specific));

		if (self->specific == NULL)
			return ENOMEM;

		err = __ihash_create (&self->specific->ht);
		if (err)
			return err;

		/* Add hash to the list of thread-specific data hashes that are
		   in use.  */
		__spin_lock (&__pthread_specific_lock);
		self->specific->next = __pthread_specific;
		self->specific->prevp = &__pthread_specific;
		if (__pthread_specific)
			__pthread_specific->prevp = &self->specific->next;
		__pthread_specific = self->specific;
		__spin_unlock (&__pthread_specific_lock);
	}

	err = __ihash_add (self->specific->ht, key, (void *) value, NULL);

	return err;
}

/* Return current value stored in the thread-specific data slot
   identified by KEY.  */
void *pthread_getspecific (pthread_key_t key)
{
	struct __pthread *self = (struct __pthread *) __pthread_self ();

	if (self->specific == NULL)
		return NULL;

	return __ihash_find (self->specific->ht, key);
}
