/* Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

/*
 *	POSIX Threads Extension: ???			<pthread.h>
 */

#ifndef _PTHREAD_H
#define _PTHREAD_H	1

#include <features.h>

#include <sched.h>
#include <time.h>

#define __need_sigset_t
#include <signal.h>

__BEGIN_DECLS

#include <bits/pthreadtypes.h>
#include <bits/pthread.h>


/* Attributes.  */

#include <bits/attr.h>

/* Initialize thread attribute object *ATTR with default attributes.  */
extern int pthread_attr_init __P ((pthread_attr_t *__attr));

/* Destroy thread attribute object *ATTR.  */
extern int pthread_attr_destroy __P ((pthread_attr_t *__attr));

/* Get `detachstate' attribute from *ATTR and return it in *DETACHSTATE.  */
extern int pthread_attr_getdetachstate __P ((__const pthread_attr_t *__attr,
					     int *__detachstate));

/* Set `detachstate' attribute in *ATTR to DETACHSTATE.  */
extern int pthread_attr_setdetachstate __P ((pthread_attr_t *__attr,
					     int __detachstate));

/* Get `schedparam' attribute from *ATTR and return it in *PARAM.  */
extern int pthread_attr_getschedparam __P ((__const pthread_attr_t *__attr,
					    struct sched_param *__param));

/* Set `schedparam' attribute in *ATTR to PARAM.  */
extern int pthread_attr_setschedparam __P ((pthread_attr_t *__attr,
				    __const struct sched_param *__param));

/* Get `stackaddr' attribute from *ATTR and return it in *STACKADDR.  */
extern int pthread_attr_getstackaddr __P ((__const pthread_attr_t *__attr,
					   void **__stackaddr));

/* Set `stackaddr' attribute in *ATTR to STACKADDR.  */
extern int pthread_attr_setstackaddr __P ((pthread_attr_t *__attr,
					   void *__stackaddr));

/* Get `stacksize' attribute from *ATTR and return it in *STACKSIZE.  */
extern int pthread_attr_getstacksize __P ((__const pthread_attr_t *__attr,
					   size_t *__stacksize));

/* Set `stacksize' attribute in *ATTR to STACKSIZE.  */ 
extern int pthread_attr_setstacksize __P ((pthread_attr_t *__attr,
					   size_t __stacksize));


/* Threads.  */

/* Create a thread with attributes given by ATTR, executing
   START_ROUTINE with argument ARG.  */
extern int
pthread_create __P ((pthread_t *__thread, __const pthread_attr_t *__attr,
		     void *(*__start_routine)(void *), void *__arg));

/* Terminate the current thread and make STATUS available to any
   thread that might join us.  */
extern void pthread_exit __P ((void *__status));

/* Make calling thread wait for termination of thread THREAD.  Return
   the exit status of the thread in *STATUS.  */
extern int pthread_join __P ((pthread_t __thread, void **__status));

/* Indicate that the storage for THREAD can be reclaimed when it
   terminates.  */
extern int pthread_detach __P ((pthread_t __thread));

/* Compare thread IDs T1 and T2.  Return nonzero if they are equal, 0
   if they are not.  */
extern int pthread_equal __P ((pthread_t __t1, pthread_t __t2));

/* Return the thread ID of the calling thread.  */
extern pthread_t pthread_self __P ((void));


/* Mutexes.  */

#include <bits/mutex.h>

/* Initialize mutex attribute object *ATTR with default attributes.  */
extern int pthread_mutexattr_init __P ((pthread_mutexattr_t *__attr));

/* Destroy thread attribute object *ATTR.  */
extern int pthread_mutexattr_destroy __P ((pthread_mutexattr_t *__attr));

/* Get `process-shared' attribute from *ATTR and return it in *PSHARED.  */
extern int
pthread_mutexattr_getpshared __P ((__const pthread_mutexattr_t * __attr,
				   int *__pshared));

/* Set `process-shared' attribute in *ATTR to PSHARED.  */
extern int pthread_mutexattr_setpshared __P ((pthread_mutexattr_t *__attr,
					      int __pshared));

/* Initialize MUTEX with the attributes specified by ATTR.  */
extern int pthread_mutex_init __P ((pthread_mutex_t *__mutex,
				    __const pthread_mutexattr_t *__attr));

/* Initializer for a mutex.  */
#define PTHREAD_MUTEX_INITIALIZER __PTHREAD_MUTEX_INITIALIZER

/* Destroy MUTEX.  */
extern int pthread_mutex_destroy __P ((pthread_mutex_t *__mutex));

/* Wait until lock for MUTEX comes available and lock it.  */
extern int pthread_mutex_lock __P ((pthread_mutex_t *__mutex));

/* Try to lock MUTEX.  */
extern int pthread_mutex_trylock __P ((pthread_mutex_t *__mutex));

/* Unlock MUTEX.  */
extern int pthread_mutex_unlock __P ((pthread_mutex_t *__mutex));


/* Condition variables.  */

#include <bits/condition.h>

/* Initialize condition variable attribute object *ATTR with default
   attributes.  */
extern int pthread_condattr_init __P ((pthread_condattr_t *__attr));

/* Destroy condition variable attribute object *ATTR.  */
extern int pthread_condattr_destroy __P ((pthread_condattr_t *__attr));

/* Get `process-shared' attribute from *ATTR and return it in *PSHARED.  */
extern int
pthread_condattr_getpshared __P ((__const pthread_condattr_t * __attr,
				  int *__pshared));

/* Set `process-shared' attribute in *ATTR to PSHARED.  */
extern int pthread_condattr_setpshared __P ((pthread_condattr_t *__attr,
					     int __pshared));

/* Initialize condition variable COND with the attributes specified by
   ATTR.  */
extern int pthread_cond_init __P ((pthread_cond_t *__cond,
				   __const pthread_condattr_t *__attr));

/* Initializer for a condition variable.  */
#define PTHREAD_COND_INITIALIZER __PTHREAD_COND_INITIALIZER

/* Destroy condition variable COND.  */
extern int pthread_cond_destroy __P ((pthread_cond_t *__cond));

/* Unblock at least one of the threads that are blocked on condition
   variable COND.  */
extern int pthread_cond_signal __P ((pthread_cond_t *__cond));

/* Unblock all threads that are blocked on condition variable COND.  */
extern int pthread_cond_broadcast __P ((pthread_cond_t *__cond));

/* Block on condition variable COND.  MUTEX should be held by the
   calling thread.  On success, MUTEX will be held by the calling
   thread.  */
extern int pthread_cond_wait __P ((pthread_cond_t *__cond,
				   pthread_mutex_t *__mutex));

/* Block on condition variable COND.  MUTEX should be held by the
   calling thread. On success, MUTEX will be held by the calling
   thread.  If the time specified by ABSTIME passes, ETIMEDOUT is
   returned, and MUTEX will nevertheless be held.  */
extern int pthread_cond_timedwait __P ((pthread_cond_t *__cond,
					pthread_mutex_t *__mutex,
					__const struct timespec *__abstime));

#ifdef __USE_GNU

/* Declare that COND should consider IMPL's waiter queue to be an
   extension of its own queue.  It is an error for either condition to
   be deallocated as long as the implication persists.  */
extern int pthread_cond_imply __P ((pthread_cond_t *__cond,
				    pthread_cond_t *__impl));

/* Declare that the implication relationship from COND to IMPL should
   cease. */
extern int pthread_cond_unimply __P ((pthread_cond_t *__cond,
				      const pthread_cond_t *__impl));

#endif


/* Once only execution.  */

#include <bits/once.h>

/* Ensure that INIT_ROUTINE has been called at most once.  */
extern int
pthread_once __P ((pthread_once_t *__once, void (*__init_routine)(void)));

/* Initializer for a once control.  */
#define PTHREAD_ONCE_INIT __PTHREAD_ONCE_INIT


/* Thread-specific data.  */

/* Create a key value identifying a location in the thread-specific
   data area.  On success the new key is stored in *KEY.  DESTRUCTOR,
   if non-NULL, is called with the value associated to that key at
   thread exit, if the associated value is non-NULL.  */
extern int pthread_key_create __P ((pthread_key_t *__key,
				    void (*__destructor)(void *)));

/* Delete KEY.  */
extern int pthread_key_delete __P ((pthread_key_t __key));

/* Store VALUE in the thread-specific data slot identified by KEY.  */
extern int pthread_setspecific __P ((pthread_key_t __key,
				     __const void *__value));

/* Return current value stored in the thread-specific data slot
   identified by KEY.  */
extern void *pthread_getspecific __P ((pthread_key_t __key));


/* Cancellation.  */

/* Special exit status that indicates cancellation.  */
#define PTHREAD_CANCELED ((void *) -1)

/* Possible values of a thread's cancelability state.  */
enum __pthread_cancelstate
{
  PTHREAD_CANCEL_ENABLE,
  PTHREAD_CANCEL_DISABLE
};

/* Possible values of a thread's cancelability type.  */
enum __pthread_canceltype
{
  PTHREAD_CANCEL_DEFERRED,
  PTHREAD_CANCEL_ASYNCHRONOUS
};

/* Request that THREAD be cancelled.  */
extern int pthread_cancel __P((pthread_t __thread));

/* Set the calling thread's cancelability state to STATE and return
   the previous cancelability state in *OLDSTATE.  */
extern int pthread_setcancelstate __P((int __state, int *__oldstate));

/* Set the calling thread's cancelability type to TYPE and return the
   previous cancelability state in OLDTYPE.  */
extern int pthread_setcanceltype __P((int __type, int *__oldtype));

/* Create a cancellation point in the calling thread.  */
extern void pthread_testcancel __P((void));

/* Push the cancellation cleanup handler ROUTINE onto the calling
   thread's cancellation cleanup stack, and prepare it to be invoked
   with the argument ARG.  */
#define pthread_cleanup_push(routine, arg)                                    \
  {                                                                           \
    struct __pthread_cleanup __buffer;                                        \
    _pthread_cleanup_push (&__buffer, (routine), (arg))

/* Remove the cancellation cleanup handler at the top of the calling
   thread's cancellation cleanup stack and invoke it if EXECUTE is
   non-zero.  */
#define pthread_cleanup_pop(execute)                                          \
    _pthread_cleanup_pop (&__buffer, (execute));                              \
  } while (0)

/* This structure is used to implement the cancellation cleanup stack.
   Never try to access its memebers directly, or use it to declare
   your own variables!  */
struct __pthread_cleanup
  {
    void (*__routine)(void *);
    void *__arg;
    struct __pthread_cleanup *__next;
  };

/* These functions are used to implement the cancellation cleanup
   stack.  Never try to call them directly!  */
extern void _pthread_cleanup_push __P((struct __pthread_cleanup *__buffer,
				       void (*__routine)(void *),
				       void *__arg));
extern void _pthread_cleanup_pop __P((struct __pthread_cleanup *__buffer,
				      int __execute));


/* Signal handling.  */

#include <bits/sigthread.h>


/* Register handlers to be called during fork(). */

/* Install handlers to be called when a new process is created with FORK.
   The PREPARE handler is called in the parent process just before performing
   FORK. The PARENT handler is called in the parent process just after FORK.
   The CHILD handler is called in the child process.  Each of the three
   handlers can be NULL, meaning that no handler needs to be called at that
   point.
   PTHREAD_ATFORK can be called several times, in which case the PREPARE
   handlers are called in LIFO order (last added with PTHREAD_ATFORK,
   first called before FORK), and the PARENT and CHILD handlers are called
   in FIFO (first added, first called).  */

extern int pthread_atfork __P((void (*__prepare) (void),
                               void (*__parent) (void),
                               void (*__child) (void)));


#ifdef __USE_EXTERN_INLINES

# ifndef _PTHREAD_H_EXTERN_INLINE
#  define _PTHREAD_H_EXTERN_INLINE extern __inline
# endif

/* Compare thread IDs T1 and T2.  Return nonzero if they are equal, 0
   if they are not.  */
_PTHREAD_H_EXTERN_INLINE int
pthread_equeal (pthread_t __t1, pthread_t __t2)
{
  return __t1 == __t2;
}

#endif /* Use extern inlines.  */

__END_DECLS

#endif /* pthread.h */
