/* Set cancelability state.
   Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <error.h>
#include <pthread.h>

#include <pt-internal.h>

/* Set the calling thread's cancelability state to STATE and return
   the previous cancelability state in *OLDSTATE.  */
int pthread_setcancelstate (int state, int *oldstate)
{
	struct __pthread *self = (struct __pthread *) __pthread_self ();

	if (state != PTHREAD_CANCEL_ENABLE && state != PTHREAD_CANCEL_DISABLE)
		return EINVAL;

	PTHREAD_CRITICAL_BEGIN;
	if (oldstate)
		*oldstate = self->cancelstate;
	self->cancelstate = state;
	PTHREAD_CRITICAL_END;

	if (self->cancel
			&& self->canceltype == PTHREAD_CANCEL_ASYNCHRONOUS
			&& self->cancelstate == PTHREAD_CANCEL_ENABLE)
		pthread_cancel ((pthread_t) self);

	return 0;
}
