/* Once control type.  Generic version.
   Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#ifndef _BITS_ONCE_H

#ifndef __need_pthread_once
# define _BITS_ONCE_H	1
#endif

#ifndef __pthread_once_defined
# if defined __need_pthread_once || defined _BITS_MUTEX_H
#  undef __need_pthread_once
#  define __pthread_once_defined

#  include <spin-lock.h>

/* Once control.  */
struct __pthread_once
{
	pthread_mutex_t __lock;
	__volatile int __done;
};

/* Initializer for a once control.  */
#  define __PTHREAD_ONCE_INIT	{ PTHREAD_MUTEX_INITIALIZER, 0 }

# endif
#endif /* Not __pthread_once_defined.  */

#ifdef _BITS_ONCE_H

#ifdef __USE_EXTERN_INLINES

# ifndef _EXTERN_INLINE
#  define _EXTERN_INLINE extern __inline
# endif

_EXTERN_INLINE int
__pthread_once (pthread_once_t *__once,
		void (*__init_routine)(void))
{
  extern int __pthread_mutex_lock (pthread_mutex_t *);
  extern int __pthread_mutex_unlock (pthread_mutex_t *);
  
  if (__once->__done)
    return 0;

  __pthread_mutex_lock (&__once->__lock);
  if (! __once->__done)
    {
      __init_routine ();
      __once->__done = 1;
    }
  __pthread_mutex_unlock (&__once->__lock);

  return 0;
}

_EXTERN_INLINE int
pthread_once (pthread_once_t *__once,
	      void (*__init_routine)(void))
{
  return __pthread_once (__once, __init_routine);
}

#endif /* Use extern inlines.  */

#endif

#endif /* bits/once.h */
