/* Initialize a condition variable.  Generic version.
   Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <errno.h>
#include <pthread.h>
#include <stdlib.h>
#include <spin-lock.h>

#include <pt-condattr.h>
#include <pt-internal.h>

/* Initialize condition variable COND with the attributes specified by
   ATTR.  */
int
pthread_cond_init (pthread_cond_t *cond,
		const pthread_condattr_t *attr)
{
	__spin_lock_init (&cond->__lock);
	cond->__queue = NULL;
	if (attr) {
		cond->__attr = (pthread_condattr_t *)
				malloc (sizeof (struct __pthread_condattr));

		if (cond->__attr == NULL)
			return ENOMEM;

		*cond->__attr = *attr;
	} else
		cond->__attr = NULL;

	return 0;
}

/* Destroy condition variable COND */
int
pthread_cond_destroy (pthread_cond_t *cond)
{
	if (!__spin_try_lock (&cond->__lock) || cond->__queue != NULL)
		return EBUSY;

	free(cond->__attr);
	free(cond->__impl);
	free(cond);
}
