/* Signal a condition. Broadcast a condition.
   Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <pthread.h>
#include <spin-lock.h>

#include <pt-internal.h>
#include <pt-condimpl.h>

static int internal_function
cond_signal (pthread_cond_t *cond, int *unblocked)
{
	struct __pthread *wakeup;
	pthread_condimpl_t *impl;

	__spin_lock (&cond->__lock);
	wakeup = __pthread_dequeue (cond->__queue);
	__spin_unlock (&cond->__lock);

	if (wakeup) {
		__pt_wakeup (wakeup);
		*unblocked = 1;
	} else {
		for (impl = cond->__impl; impl && *unblocked == 0; impl = impl->next)
			cond_signal (impl->cond, unblocked);
	}

	return 0;
}

/* Unblock at least one of the threads that are blocked on condition
   variable COND.  */
int pthread_cond_signal (pthread_cond_t * cond)
{
	int unblocked = 0;

	return cond_signal (cond, &unblocked);
}

/* Unblock all threads that are blocked on condition variable COND.  */
int pthread_cond_broadcast (pthread_cond_t * cond)
{
	struct __pthread *wakeup;
	pthread_condimpl_t *impl;

	__spin_lock (&cond->__lock);
	wakeup = cond->__queue;
	while (wakeup) {
		wakeup->prevp = NULL;
		wakeup = wakeup->next;
	}
	wakeup = cond->__queue;
	cond->__queue = NULL;
	__spin_unlock (&cond->__lock);

	while (wakeup) {
		__pt_wakeup (wakeup);
		wakeup = wakeup->next;
	}

	for (impl = cond->__impl; impl; impl = impl->next)
		pthread_cond_broadcast (impl->cond);

	return 0;
}
