/* Initialize a condition attribute.  Generic version.
   Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <errno.h>
#include <pthread.h>
#include <stdlib.h>
#include <spin-lock.h>

#include <pt-condattr.h>

/* Initialize condition variable attributes object ATTR */
int
pthread_condattr_init (pthread_condattr_t *attr)
{
	if (attr == NULL)
		return EINVAL;

	attr = (pthread_condattr_t *)malloc(sizeof (struct __pthread_condattr));
	if(attr == NULL)
		return ENOMEM;
	attr->pshared = PTHREAD_PROCESS_PRIVATE;

	return 0;
}

/* Destroy condition variable attributes object ATTR */
int
pthread_condattr_destroy (pthread_condattr_t *attr)
{
	free(attr);
	return 0;
}

/* Store the process-shared attribute of ATTR in *PSHARED */
int
pthread_condattr_getpshared (const pthread_condattr_t *attr, int *pshared)
{
	if(attr) {
		*pshared = attr->pshared;
		return 0;
	} else
		return EINVAL;
}

/* Set the process-shared attribute of ATTR specified in PSHARED */
int
pthread_condattr_setpshared (pthread_condattr_t *attr, int pshared)
{
	if(attr) {
		switch(pshared)
		{
		case PTHREAD_PROCESS_SHARED:
		case PTHREAD_PROCESS_PRIVATE:
			attr->pshared = pshared;
			return 0;
		}
	}
	return EINVAL;
}
