/* Initialize a mutex.  Generic version.
   Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <errno.h>
#include <pthread.h>
#include <stdlib.h>
#include <spin-lock.h>

#include <mutex.h>
#include <pt-mutexattr.h>
#include <pt-internal.h>

/* Initialize MUTEX with the attributes specified by ATTR.  */
int
__pthread_mutex_init (pthread_mutex_t *mutex,
		const pthread_mutexattr_t *attr)
{
	__spin_lock_init (&mutex->__held);
	__spin_lock_init (&mutex->__lock);
	mutex->__queue = NULL;
	if (attr) {
		int save_errno = errno;

		mutex->__attr = (pthread_mutexattr_t *)
				malloc (sizeof (struct __pthread_mutexattr));

		if (mutex->__attr == NULL) {
			__set_errno (save_errno);
			return ENOMEM;
		}
		*mutex->__attr = *attr;
	} else
		mutex->__attr = NULL;

	return 0;
}

weak_alias (__pthread_mutex_init, pthread_mutex_init);
