/* Lock and unlock a mutex.  Generic version.
   Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <errno.h>
#include <pthread.h>
#include <spin-lock.h>

#include <pt-internal.h>

/* Lock MUTEX, block if we can't get it.  */
int __pthread_mutex_lock_solid (pthread_mutex_t *mutex)
{
	struct __pthread *self = (struct __pthread *) __pthread_self ();

	__spin_lock (&mutex->__lock);
	if (__spin_try_lock (&mutex->__held)) {
		__spin_unlock (&mutex->__lock);
		return 0;
	}

	/* Add ourselves to the queue.  */
	self->next = mutex->__queue;
	self->prevp = &mutex->__queue;
	if (mutex->__queue)
		mutex->__queue->prevp = &self->next;
	mutex->__queue = self;
	__spin_unlock (&mutex->__lock);

	/* Block the thread.  */
	__pt_block (self);

	return 0;
}

/* Lock MUTEX, return an error if we can't get it.  */
int __pthread_mutex_trylock_solid (pthread_mutex_t *mutex)
{
	return (__spin_try_lock (&mutex->__held) ? 0 : EBUSY);
}

/* Unlock MUTEX, rescheduling a waiting thread.  */
int __pthread_mutex_unlock (pthread_mutex_t *mutex)
{
	struct __pthread *wakeup;

	__spin_lock (&mutex->__lock);
	if (mutex->__queue == NULL) {
		__spin_unlock (&mutex->__held);
		__spin_unlock (&mutex->__lock);
		return 0;
	}
	wakeup = mutex->__queue;
	if (wakeup) {
		if (wakeup->next)
			wakeup->next->prevp = wakeup->prevp;
		*wakeup->prevp = wakeup->next;
	}
	__spin_unlock (&mutex->__lock);

	if (wakeup)
		__pt_wakeup (wakeup);

	return 0;
}

weak_alias (__pthread_mutex_unlock, pthread_mutex_unlock)
