/* Enqueue and dequeue a thread. Remove a thread from queue.
   Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <assert.h>

#include <pt-internal.h>

/* Enqueue THREAD on QUEUE.  */
void __pthread_enqueue (struct __pthread **queue, struct __pthread *thread)
{
	thread->next = *queue;
	thread->prevp = queue;
	if (*queue)
		(*queue)->prevp = &thread->next;
	*queue = thread;
}

/* Dequeue a thread from QUEUE.  */
struct __pthread *__pthread_dequeue (struct __pthread *queue)
{
	struct __pthread *thread;

	/* XXX We should dequeue threads according to the scheduling
	   policy.  And even when all threads have the same priority, we
	   should probably do FIFO instead of LIFO.  */
	thread = queue;
	if (thread) {
		if (thread->next)
			thread->next->prevp = thread->prevp;
		*thread->prevp = thread->next;
		thread->prevp = NULL;
	}

	return thread;
}

/* Remove THREAD from the queue it is on.  */
void __pthread_remove (struct __pthread *thread)
{
	assert (thread->prevp);

	if (thread->next)
		thread->next->prevp = thread->prevp;
	*thread->prevp = thread->next;
	thread->prevp = NULL;
}
