/* libc-internal interface for mutex locks.  Hurd version.
   Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#ifndef _BITS_LIBC_LOCK_H
#define _BITS_LIBC_LOCK_H 1

#include <pthread.h>

/* Type of mutex lock.  */
typedef pthread_mutex_t __libc_lock_t;

/* Define a lock variable NAME with storage class CLASS.  The lock must be
   initialized with __libc_lock_init before it can be used (or define it
   with __libc_lock_define_initialized, below).  Use `extern' for CLASS to
   declare a lock defined in another module.  In public structure
   definitions you must use a pointer to the lock structure (i.e., NAME
   begins with a `*'), because its storage size will not be known outside
   of libc.  */
#define __libc_lock_define(CLASS,NAME) \
  CLASS __libc_lock_t NAME;

/* Define an initialized lock variable NAME with storage class CLASS.  */
#define __libc_lock_define_initialized(CLASS,NAME) \
 CLASS __libc_lock_t NAME = PTHREAD_MUTEX_INITIALIZER;

/* Initialize the named lock variable, leaving it in a consistent, unlocked
   state.  */
#define __libc_lock_init(NAME) __pthread_mutex_init (&(NAME), NULL)

/* Finalize the named lock variable, which must be locked.  It cannot be
   used again until __libc_lock_init is called again on it.  This must be
   called on a lock variable before the containing storage is reused.  */
#define __libc_lock_fini(NAME) __pthread_mutex_destroy (&(NAME))

/* Lock the named lock variable.  */
#define __libc_lock_lock(NAME) __pthread_mutex_lock (&(NAME))

/* Lock the named lock variable.  */
#define __libc_lock_trylock(NAME) __pthread_mutex_trylock (&(NAME))

/* Unlock the named lock variable.  */
#define __libc_lock_unlock(NAME) __pthread_mutex_unlock (&(NAME))

/* Define once control variable.  */
#define __libc_once_define(CLASS,NAME) \
  CLASS pthread_once_t NAME = PTHREAD_ONCE_INIT

/* Call handler iff the first call.  */
#define __libc_once(ONCE_CONTROL, INIT_FUNCTION) \
  __pthread_once (&(ONCE_CONTROL), (INIT_FUNCTION))


/* Start a critical region with a cleanup function */
#define __libc_cleanup_region_start(FCT, ARG)				    \
{									    \
  typeof (FCT) __save_FCT = FCT;					    \
  typeof (ARG) __save_ARG = ARG;					    \
  /* close brace is in __libc_cleanup_region_end below. */

/* End a critical region started with __libc_cleanup_region_start. */
#define __libc_cleanup_region_end(DOIT)					    \
  if (DOIT)								    \
    (*__save_FCT)(__save_ARG);						    \
}


/* We need portable names for some functions.  */
#define __libc_mutex_unlock __pthread_mutex_unlock

/* Type for key of thread specific data.  */
typedef pthread_key_t __libc_key_t;

/* Create key for thread specific data.  */
#define __libc_key_create(KEY,DEST) __pthread_key_create ((KEY), (DEST))

/* Set thread-specific data associated with KEY to VAL.  */
#define __libc_setspecific(KEY,VAL) __pthread_setspecific ((KEY), (VAL))

/* Get thread-specific data associated with KEY.  */
#define __libc_getspecific(KEY) __pthread_getspecific ((KEY))


/* XXX Until we support recursive locks.  */
#define __libc_lock_define_initialized_recursive __libc_lock_define_initialized
#define __libc_lock_init_recursive __libc_lock_init
#define __libc_lock_fini_recursive __libc_lock_fini
#define __libc_lock_trylock_recursive __libc_lock_trylock
#define __libc_lock_unlock_recursive __libc_lock_unlock
#define __libc_lock_lock_recursive __libc_lock_lock

#endif	/* bits/libc-lock.h */
