/* Setup thread stack.  Hurd/i386 version.
   Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <pt-internal.h>

/* The stack layout used on the i386 is:

   +-----------------+  <- stack top
   |                 |
   |  Fast TSD       |
   |                 |
   +-----------------+
   |  ARG            |
   +-----------------+
   |  START_ROUTINE  |
   +-----------------+  | stack growth
   |  0              |  v
   +-----------------+

    On the Hurd, We reserve __hurd_threadvar_max `unsigned long int's'
    of (fast) thread-specific data (TSD).  */

/* Set up the stack for THREAD, such that it appears as if
   START_ROUTINE and ARG were passed to the new thread's entry-point.
   Return the stack pointer for the new thread.  */
void *__pt_stack_setup (struct __pthread *thread,
		void *(*start_routine) (void *), void *arg)
{
	int *top;

	/* We cannot use __hurd_threadvar_location_from_sp here since
	   __hurd_threadvar_stack_mask and __hurd_threadvar_stack_offset
	   might not be initialized yet.  */
#if defined (STACK_GROWTH_DOWN)
	/* Calculate top of the new stack.  */
	top = (int *) (thread->attr.stackaddr + thread->attr.stacksize);
	top -= __hurd_threadvar_max;
	top[_HURD_THREADVAR_THREAD] = (int) thread;
	if (start_routine) {
		/* Set up call frame.  */
		*--top = (int) arg;	/* Argument to START_ROUTINE.  */
		*--top = (int) start_routine;
		*--top = (int) 0;	/* Fake return address.  */
	}
#elif defined (STACK_GROWTH_UP)
	top[_HURD_THREADVAR_THREAD] = (int) thread;
	top += __hurd_threadvar_max;
	if (start_routine) {
		/* Set up call frame.  */
		*++top = (int) arg;	/* Argument to START_ROUTINE.  */
		*++top = (int) start_routine;
		*++top = (int) 0;	/* Fake return address.  */
	}
#else
#	error stack direction unknown
#endif

	return top;
}
