/* Stub versions of mutex handling functions.
   Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <errno.h>
#include <pthread.h>
#include <spin-lock.h>

/* If no thread library is linked in, these functions will be used to
   handle `mutexes'.  They use only spin locks.  */

int
__pthread_mutex_init (pthread_mutex_t *mutex,
		const pthread_mutexattr_t *attr)
{
	__spin_lock_init (&mutex->__held);
	return 0;
}

int __pthread_mutex_destroy (pthread_mutex_t *mutex)
{
	__spin_lock_init (&mutex->__held);
	return 0;
}

int __pthread_mutex_lock_solid (pthread_mutex_t *mutex)
{
	__spin_lock_solid (&mutex->__held);
	return 0;
}

strong_alias (__pthread_mutex_lock_solid, __pthread_mutex_lock)

int
__pthread_mutex_trylock_solid (pthread_mutex_t *mutex)
{
	return (__spin_try_lock (&mutex->__held) ? 0 : EBUSY);
}

strong_alias (__pthread_mutex_trylock_solid, __pthread_mutex_trylock)

int
__pthread_mutex_unlock (pthread_mutex_t *mutex)
{
	__spin_unlock (&mutex->__held);
	return 0;
}
