/* Send a signal to a thread.
   Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <errno.h>
#include <pthread.h>
#include <hurd.h>
#include <hurd/signal.h>

#include <pt-internal.h>

/* Send signal SIGNO to the given thread.  */
int pthread_kill (pthread_t thread, int signo)
{
	struct hurd_signal_detail d;
	struct hurd_sigstate *ss;

#define thread ((struct __pthread *) thread)

	if (signo == 0)
		return 0;

	if (signo < 0 || signo >= NSIG)
		return EINVAL;

	/* SIGKILL is not deliverd as a normal signal.
	   Sending SIGKILL to a process means to terminate its task.  */
	if (signo == SIGKILL)
		raise (SIGKILL);

	d.code = 0;
	d.exc = 0;

	ss = _hurd_thread_sigstate (thread->kernel_thread);
	__spin_lock (&ss->lock);

	/* Raise the signal on the target thread.  */
	_hurd_raise_signal (ss, signo, &d);

	return 0;
#undef thread
}
