/* Block a thread for a period of time.  Mach version.
   Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <assert.h>
#include <errno.h>
#include <sys/time.h>

#include <mach.h>
#include <mach/message.h>

#include <pt-internal.h>

/* Block THREAD.  Automagically unblock if ABSTIME passes.  */
int __pt_timedblock (struct __pthread *thread, const struct timespec *abstime)
{
	struct timeval cur_time, abs_time;
	struct timeval time;
	mach_msg_timeout_t timeout = 0;
	mach_msg_header_t msg;
	error_t err;

	if (abstime) {
		TIMESPEC_TO_TIMEVAL (&abs_time, abstime);
		gettimeofday (&cur_time, NULL);

		if (timercmp (&cur_time, &abs_time, >))
			return ETIMEDOUT;

		timersub (&abs_time, &cur_time, &time);
		/* XXX Handle overflow.  */
		timeout = time.tv_sec * 1000000 + time.tv_usec;
	}

	err = __mach_msg (&msg, MACH_RCV_MSG | MACH_RCV_TIMEOUT
			| MACH_RCV_INTERRUPT, 0, sizeof msg,
			thread->wakeupmsg.msgh_remote_port,
			timeout, MACH_PORT_NULL);
	return (err == MACH_RCV_TIMED_OUT ? ETIMEDOUT : 0);
}
